package examples.xml.viewer;

import com.ibm.mqe.*;
import com.ibm.mqe.adapters.*;
import com.ibm.mqe.attributes.*;
import java.io.*;
import com.ibm.mqe.xml.*;
import com.ibm.mqe.messagestore.*;

// Version mqe_xml_supportpac/source/examples/xml/viewer/View.java, MQeXML, la000 1.2
/* -----------------------------------------------------------------------------*/
/*                   MQSeries Everyplace for Multiplatforms                     */
/*                                                                              */
/* Licensed Materials - Property of IBM                                         */
/*                                                                              */
/* 5765-E63                                                                     */
/*                                                                              */
/* (C) Copyright IBM Corp. 1999                                                 */
/* -----------------------------------------------------------------------------*/

/**
* To read a message from the disk, and output a textual representation of it in XML.
*
* <B>Syntax</B>
* <TT>java examples.xml.messageviewer.View &lt;message_file_name&gt; [ -unwrapcompressor &lt;compressor&gt; ]</TT>
*
* <DL>
* <DT>message_file_name</DT>
* <DD>The message file in that directory, which you want to decode into XML.
* This can be a fully-qualified path and file name, or a path and file name 
* qualified relative to the current directory.</DD>
* <DT>unwrap_compressor</DT>
* <DD>An optional parameter, followed by the name of the compressor to use during the unwrap operation.
* Supported values of &lt;compressor&gt; above are :
*       <UL>
*           <LI>lzw</LI>
*           <LI>null</LI>
*       </UL>
* <p>When the message being decoded has a wrapped message enclosed by the outer message.
* Remove the wrapped part of the message, unwrap it using an attribute containing the 
* specified compressor, and re-add the unwrapped part of the message.</p>
* <p>If the -unwrapcompressor value is not supplied, then the wrapped part of the message is
* rendered to XML as a byte array.</p>
* <p>Note: Case IS sensitive for these parameters.</p>
* </DD>
* </DL>
*
*/
public class View implements Errorhandler {

    /**
    * Read in an unencrypted and uncompressed MQe message, and render it to XML on the stdout stream.
    */
    public static void main( String args[] ) {
        try {

            View me = new View();
            me.activate( args );

        } catch( Exception e) {
            System.err.println("Failure!. "+e.toString() );
            // e.printStackTrace();
        }    
    }

    /**
    * If we get a syntax error, this string will be put out.
    */
    private static String syntax = 
        "Syntax:"
      + "\njava examples.xml.viewer.View <file_name>"
      + "\n"
      + "\nParameters:"
      + "\n    file_name                           The name of the message file within that directory."
      + "\n    [ -unwrapcompressor (lzw|null) ]    Optional name of compressor to unwrap parts of msg with."
      + "\n"
      + "\nTip:"
      + "\nYou can enumerate through a complete MQe registry directory, rendering files using:"
      + "\nWindows syntax:"
      + "\nFOR /R <my_qmgr_directory> %i DO (*) java examples.xml.viewer.View %i"
      ;
	
	/**
    * Do the work of this class.
	*/
	public void   activate( String args[] ) throws Exception {
    
        MQeCompressor compressor = null ;
        MQeAttribute  attribute  = null ;
        boolean unwrapWrappedMessages = false ;

        if( args.length < 1 ) {
            throw new Exception("Too few parameters!\n"+syntax);
        }
                               
        String sourceFileName = args[0];

        int param = 1 ; // Which parameter are we looking at right now.
        while( param < args.length ) {

            String thisParam = args[param];
            if( thisParam.equals("-unwrapcompressor") ) {
                param +=1 ; // We've processed this parameter.
                if( param >=  args.length ) {
                    throw new Exception("Flag -unwrapcompressor must be followed by option string.\n"
                                       +syntax);
                }
                thisParam = args[param];
                if( thisParam.equals("null") ) {
                    compressor = null ;
                    unwrapWrappedMessages = true ;
                } else {
                    if ( thisParam.equals("lzw") ) {
                        compressor = new MQeLZWCompressor();
                        unwrapWrappedMessages = true ;
                    } else {
                        throw new Exception("Flag -unwrapcompressor followed by unrecognised value ("
                                            +thisParam+")\n"+syntax);
                    }
                }
            }

            param +=1 ; // Move onto the next parameter.
        }

        // Only create an attribute if the compressor has not been specified.
        if( compressor != null ) {
            attribute = new MQeAttribute(null, null, compressor);
        }

        // Remove any relative-parts of the directory path, resolving them to absolute path info.
        File sourceFile = new File( sourceFileName ); // The filename the user provided.
        File fullSourceFile       = sourceFile.getCanonicalFile(); 

        System.out.println("<DecodedFile name=\""+args[0]+">");

        String baseDirectory        = fullSourceFile.getParent();
        String fileInBaseDirectory  = fullSourceFile.getName();

        // Open a disk fields adapter.
        MQeAdapter adapter = new MQeDiskFieldsAdapter();

        // Activate the adapter to read from a set directory.
        adapter.activate( baseDirectory , null , MQe.MQe_Adapter_UPDATE , 0 , 0 ); 

        MQeFields fields = null ;

        // Open the message file...
        try {
            adapter.open( fileInBaseDirectory ); 
    
            // Read the file into a fields structure.
            fields = (MQeFields) adapter.readObject( null );
        } catch ( Exception e ) {
            System.out.println("<FieldsReadError comment=\"file is not a dumped MQeFields object\">"
                              +sourceFileName+"</FieldsReadError>" );      
            System.out.println("</DecodedFile>");
            throw new Exception("File "+sourceFileName+" is not a valid dumped MQeFields object.");
        }

        // If there is a wrapped field there, then unwrap it.
        if( fields instanceof MQeMsgObject ) {

            // Only unwrap any wrapped messages if the user specified an attribute to do so.
            if( unwrapWrappedMessages ) {
                if( fields.contains(MQe.Msg_WrapMsg) ) {
                    MQeMsgObject wrapper    = (MQeMsgObject) fields;
                    MQeMsgObject wrappedMsg = null ;
                    try {
                        wrappedMsg = wrapper.unwrapMsgObject( attribute );
                        // wrappedMsg = wrapper.unwrapMsgObject( null );
                    } catch( Exception e1 ) {
                        // Failed to unwrap the message object. Leave it as-is in fields.
                        // System.err.println("Failed to unwrap the wrapped part of the message. "+e1.toString());
                        // e1.printStackTrace();
                    }
    
                    if( null != wrappedMsg ) {
                        // Clone the original, so we can restore it if things go wrong.
                        MQeFields cloneOfOriginal = (MQeFields)fields.clone();
    
                        try {
                            // We have unwrapped the contents of the wrapped message.
                            // So remove the wrapped byte array message object, and replace with
                            // a fields structure...
                            wrapper.delete( MQe.Msg_WrapMsg );
                            wrapper.putFields( "UnwrappedMsg" , (MQeFields) wrappedMsg );
                        } catch( Exception e2 ) {
                            // System.err.println("Failed to expand the wrapped part of the message. "+e2.toString());
                            // e2.printStackTrace();
                            // Something has gone wrong. 
                            // Restore the fields to it's former self...
                            fields = cloneOfOriginal ;
                        }
                   }
                }
            }
        } else {
            // System.err.println("It's not a message object, so we're not expanding wrapped messages.");
        }

        // Create a converter tool...
    	MQeXMLConverter converter = new MQeXMLConverter(this);

        // Convert the fields message into an XML string...
        String results = converter.toString( fields );

        // Output the results...
        System.out.println( results );

        // End the decodedFile xml tag...
        System.out.println("</DecodedFile>");
	}

    /**
    * Output any errors from the XML converter to the stderr output stream.
    */
    public void error(String error, Exception e) {
        System.err.println(error+"  "+e.toString() );
        e.printStackTrace();
    }
}
